@@ -69,8 +69,8 @@ addCondition <- function(condDefs, dtOld, newvar) {
 
   for (i in 1:nrow(condDefs)) {
 
-    evalDef(newvar, cDefs[i,formula], cDefs[i,dist], chkVars)
-    evalDef(newvar, cDefs[i,condition], cDefs[i,dist], chkVars)
+    .evalDef(newvar, cDefs[i,formula], cDefs[i,dist], chkVars)
+    .evalDef(newvar, cDefs[i,condition], cDefs[i,dist], chkVars)
 
   }
 
@@ -91,7 +91,7 @@ addCondition <- function(condDefs, dtOld, newvar) {
     dtTemp <- dtOld[eval(parse(text = condition) )]
     n = nrow(dtTemp)
 
-    dtTemp <- generate(cDefs[i,], n, dtTemp, oldkey)
+    dtTemp <- .generate(cDefs[i,], n, dtTemp, oldkey)
 
     dtTemp <- data.table::data.table(dtTemp)
     dtTemp <- dtTemp[, list(get(oldkey), get(newvar))]
